pub mod admin;
pub mod auth;
pub mod filters;
pub mod fs;
pub mod user;

use std::io::Write;

use tiny_http::{Response, ResponseBox};

struct ChannelReader(std::sync::mpsc::Receiver<String>);

impl std::io::Read for ChannelReader {
    fn read(&mut self, mut buf: &mut [u8]) -> std::io::Result<usize> {
        self.0.recv().map(|s| buf.write(s.as_bytes())).unwrap_or(Ok(0))
    }
}

pub fn header(name: &str, data: &str) -> tiny_http::Header { tiny_http::Header::from_bytes(name, data).unwrap() }

pub fn get_reply<T>(data: &T) -> Result<ResponseBox, AppError>
where T: serde::Serialize {
    Ok(Response::from_data(serde_json::to_vec(data).unwrap())
        .with_header(header("content-type", "application/json; charset=utf-8"))
        .boxed())
}

#[derive(thiserror::Error, Debug, Clone)]
pub enum AppError {
    #[error("unauthorized")]
    Unauthorized(&'static str),
    #[error("forbidden")]
    Forbidden(&'static str),
    #[error("bad request")]
    BadRequest(&'static str),
    #[error("not found")]
    NotFound,
    #[allow(dead_code)]
    #[error("internal error")]
    InternalError(&'static str)
}

impl AppError {
    pub fn err<T>(&self) -> Result<T, AppError> { Err(self.clone()) }
}
